﻿/*
 * Copyright (C)  2011  Axel Kesseler
 * 
 * This software is free and you can use it for any purpose. Furthermore, 
 * you are free to copy, to modify and/or to redistribute this software.
 * 
 * In addition, this software is distributed in the hope that it will be 
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * 
 */

using System;
using System.Text;
using System.Diagnostics;
using System.Windows.Forms;
using System.ComponentModel;
using System.Collections.Generic;
using System.Runtime.InteropServices;

namespace plexdata.Utilities.Clipboard
{
    public class NativeClipboard : IDisposable
    {
        private bool opened = false;

        public NativeClipboard()
            : base()
        {
            this.Open(NativeClipboard.GetDesktopWindow());
        }

        public NativeClipboard(Control parent)
            : base()
        {
            if (parent == null)
            {
                throw new NullReferenceException("Parent control cannot be null!");
            }
            this.Open(parent.Handle);
        }

        public NativeClipboard(IntPtr hParent)
            : base()
        {
            this.Open(hParent);
        }

        public void Dispose()
        {
            this.Close();
        }

        public bool IsOpened()
        {
            return this.opened;
        }

        public bool Open()
        {
            return this.Open(NativeClipboard.GetDesktopWindow());
        }

        public bool Open(IntPtr hParent)
        {
            if (!this.opened)
            {
                if (hParent == IntPtr.Zero)
                {
                    throw new NullReferenceException("Parent window handle cannot be null!");
                }
                this.opened = NativeClipboard.OpenClipboard(hParent);
            }
            return this.opened;
        }

        public bool Reopen()
        {
            return this.Reopen(NativeClipboard.GetDesktopWindow());
        }

        public bool Reopen(IntPtr hParent)
        {
            if (this.opened)
            {
                if (!this.Close()) { return false; }
            }

            return this.Open(hParent);
        }

        public bool Empty()
        {
            if (!this.opened)
            {
                throw new InvalidOperationException("Clipboard is not opened!");
            }

            return NativeClipboard.EmptyClipboard();
        }

        public bool Close()
        {
            bool success = true;
            if (this.opened)
            {
                success = NativeClipboard.CloseClipboard();
                this.opened = false;
            }
            return success;
        }

        public bool HasData()
        {
            if (!this.opened)
            {
                throw new InvalidOperationException("Clipboard is not opened!");
            }
            return (0 != NativeClipboard.EnumClipboardFormats(0));
        }

        public int[] GetFormats()
        {
            if (!this.opened)
            {
                throw new InvalidOperationException("Clipboard is not opened!");
            }

            List<int> result = new List<int>();
            int format = NativeClipboard.EnumClipboardFormats(0);
            while (format != 0)
            {
                result.Add(format);
                format = NativeClipboard.EnumClipboardFormats(format);
            }
            return result.ToArray(); ;
        }

        public int GetSize(int format)
        {
            if (!this.opened)
            {
                throw new InvalidOperationException("Clipboard is not opened!");
            }

            int result = 0;
            IntPtr hGlobal = IntPtr.Zero;
            try
            {
                hGlobal = NativeClipboard.GetClipboardData(format);
                if (hGlobal != IntPtr.Zero)
                {
                    IntPtr size = NativeClipboard.GlobalSize(hGlobal);
                    result = size.ToInt32();
                }
            }
            finally
            {
                if (hGlobal == IntPtr.Zero)
                {
                    NativeClipboard.GlobalUnlock(hGlobal);
                }
            }
            return result;
        }

        public string GetName(int format)
        {
            return this.GetName(format, true);
        }

        public string GetName(int format, bool friendly)
        {
            if (!this.opened)
            {
                throw new InvalidOperationException("Clipboard is not opened!");
            }

            #region Handle standard clipboard format names.
            switch (format)
            {
                case NativeClipboard.CF_TEXT:
                    return (friendly ? "Text" : "CF_TEXT");
                case NativeClipboard.CF_BITMAP:
                    return (friendly ? "Bitmap" : "CF_BITMAP");
                case NativeClipboard.CF_METAFILEPICT:
                    return (friendly ? "Metafile" : "CF_METAFILEPICT");
                case NativeClipboard.CF_SYLK:
                    return (friendly ? "Sylk" : "CF_SYLK");
                case NativeClipboard.CF_DIF:
                    return (friendly ? "Dif" : "CF_DIF");
                case NativeClipboard.CF_TIFF:
                    return (friendly ? "Tiff" : "CF_TIFF");
                case NativeClipboard.CF_OEMTEXT:
                    return (friendly ? "OEM Text" : "CF_OEMTEXT");
                case NativeClipboard.CF_DIB:
                    return (friendly ? "DIB" : "CF_DIB");
                case NativeClipboard.CF_PALETTE:
                    return (friendly ? "Palette" : "CF_PALETTE");
                case NativeClipboard.CF_PENDATA:
                    return (friendly ? "Pen Data" : "CF_PENDATA");
                case NativeClipboard.CF_RIFF:
                    return (friendly ? "Riff" : "CF_RIFF");
                case NativeClipboard.CF_WAVE:
                    return (friendly ? "Wave" : "CF_WAVE");
                case NativeClipboard.CF_UNICODETEXT:
                    return (friendly ? "Unicode Text" : "CF_UNICODETEXT");
                case NativeClipboard.CF_ENHMETAFILE:
                    return (friendly ? "Enhanced Metafile" : "CF_ENHMETAFILE");
                case NativeClipboard.CF_HDROP:
                    return (friendly ? "Drop Handle" : "CF_HDROP");
                case NativeClipboard.CF_LOCALE:
                    return (friendly ? "Locale" : "CF_LOCALE");
                case NativeClipboard.CF_DIBV5:
                    return (friendly ? "DIB V5" : "CF_DIBV5");
                case NativeClipboard.CF_MAX:
                    return (friendly ? "Max" : "CF_MAX");
                case NativeClipboard.CF_OWNERDISPLAY:
                    return (friendly ? "Owner Display" : "CF_OWNERDISPLAY");
                case NativeClipboard.CF_DSPTEXT:
                    return (friendly ? "DSP Text" : "CF_DSPTEXT");
                case NativeClipboard.CF_DSPBITMAP:
                    return (friendly ? "DSP Bitmap" : "CF_DSPBITMAP");
                case NativeClipboard.CF_DSPMETAFILEPICT:
                    return (friendly ? "DSP Metafile" : "CF_DSPMETAFILEPICT");
                case NativeClipboard.CF_DSPENHMETAFILE:
                    return (friendly ? "DSP Enhanced Metafile" : "CF_DSPENHMETAFILE");
                case NativeClipboard.CF_PRIVATEFIRST:
                    return (friendly ? "Private First" : "CF_PRIVATEFIRST");
                case NativeClipboard.CF_PRIVATELAST:
                    return (friendly ? "Private Last" : "CF_PRIVATELAST");
                case NativeClipboard.CF_GDIOBJFIRST:
                    return (friendly ? "GDI Object First" : "CF_GDIOBJFIRST");
                case NativeClipboard.CF_GDIOBJLAST:
                    return (friendly ? "GDI Object Last" : "CF_GDIOBJLAST");
            }
            #endregion // Handle standard clipboard format names.

            StringBuilder buffer = new StringBuilder(256 * sizeof(char));
            if (0 != NativeClipboard.GetClipboardFormatName(format, buffer, buffer.Capacity))
            {
                return buffer.ToString();
            }
            else
            {
                return "<unknown>";
            }
        }

        public byte[] GetData(int format)
        {
            if (!this.opened)
            {
                throw new InvalidOperationException("Clipboard is not opened!");
            }

            byte[] result = null;
            IntPtr hGlobal = IntPtr.Zero;
            try
            {
                hGlobal = NativeClipboard.GetClipboardData(format);
                if (hGlobal != IntPtr.Zero)
                {
                    IntPtr size = NativeClipboard.GlobalSize(hGlobal);
                    if (size.ToInt32() > 0)
                    {
                        IntPtr pGlobal = NativeClipboard.GlobalLock(hGlobal);
                        if (pGlobal != IntPtr.Zero)
                        {
                            result = new byte[size.ToInt32()];
                            Marshal.Copy(pGlobal, result, 0, result.Length);
                        }
                    }
                }
            }
            finally
            {
                if (hGlobal == IntPtr.Zero)
                {
                    NativeClipboard.GlobalUnlock(hGlobal);
                }
            }
            return result;
        }

        public bool IsString(int format)
        {
            switch (format)
            {
                case NativeClipboard.CF_TEXT:
                case NativeClipboard.CF_OEMTEXT:
                case NativeClipboard.CF_UNICODETEXT:
                    return true;
                default:
                    return false;
            }
        }

        public string GetString(int format, byte[] buffer)
        {
            string result = "";
            if (this.IsString(format))
            {
                switch (format)
                {
                    case NativeClipboard.CF_TEXT:
                    case NativeClipboard.CF_OEMTEXT:
                        result = Encoding.Default.GetString(buffer);
                        break;
                    case NativeClipboard.CF_UNICODETEXT:
                        result = Encoding.Unicode.GetString(buffer);
                        break;
                    default:
                        return "";
                }
            }
            return result;
        }

        #region Native clipboard imports.

        [DllImport("kernel32.dll", SetLastError = true)]
        private static extern IntPtr GlobalSize(IntPtr hGlobal);

        [DllImport("kernel32.dll", SetLastError = true)]
        private static extern IntPtr GlobalLock(IntPtr hGlobal);

        [DllImport("kernel32.dll", SetLastError = true)]
        private static extern bool GlobalUnlock(IntPtr hGlobal);

        [DllImport("user32.dll", SetLastError = true)]
        private static extern bool OpenClipboard(IntPtr hParent);

        [DllImport("user32.dll", SetLastError = true)]
        private static extern bool CloseClipboard();

        [DllImport("user32.dll", SetLastError = true)]
        private static extern int EnumClipboardFormats(int format);

        [DllImport("user32.dll", SetLastError = true, CharSet = CharSet.Unicode)]
        private static extern int GetClipboardFormatName(int format, [Out] StringBuilder name, int count);

        [DllImport("user32.dll", SetLastError = true)]
        private static extern bool EmptyClipboard();

        [DllImport("user32.dll", SetLastError = true)]
        private static extern IntPtr GetClipboardData(int format);

        [DllImport("user32.dll", SetLastError = true)]
        private static extern IntPtr GetDesktopWindow();

        #endregion // Native clipboard imports.

        #region Native clipboard constant definitions.

        private const int CF_TEXT = 1;
        private const int CF_BITMAP = 2;
        private const int CF_METAFILEPICT = 3;
        private const int CF_SYLK = 4;
        private const int CF_DIF = 5;
        private const int CF_TIFF = 6;
        private const int CF_OEMTEXT = 7;
        private const int CF_DIB = 8;
        private const int CF_PALETTE = 9;
        private const int CF_PENDATA = 10;
        private const int CF_RIFF = 11;
        private const int CF_WAVE = 12;
        private const int CF_UNICODETEXT = 13;
        private const int CF_ENHMETAFILE = 14;
        private const int CF_HDROP = 15;
        private const int CF_LOCALE = 16;
        private const int CF_DIBV5 = 17;
        private const int CF_MAX = 18;
        private const int CF_OWNERDISPLAY = 0x0080;
        private const int CF_DSPTEXT = 0x0081;
        private const int CF_DSPBITMAP = 0x0082;
        private const int CF_DSPMETAFILEPICT = 0x0083;
        private const int CF_DSPENHMETAFILE = 0x008E;
        private const int CF_PRIVATEFIRST = 0x0200;
        private const int CF_PRIVATELAST = 0x02FF;
        private const int CF_GDIOBJFIRST = 0x0300;
        private const int CF_GDIOBJLAST = 0x03FF;

        #endregion // Native clipboard constant definitions.
    }
}
