﻿/*
 * Copyright (C)  2011  Axel Kesseler
 * 
 * This software is free and you can use it for any purpose. Furthermore, 
 * you are free to copy, to modify and/or to redistribute this software.
 * 
 * In addition, this software is distributed in the hope that it will be 
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * 
 */

using System;
using System.IO;
using System.Xml;
using System.Drawing;
using System.Diagnostics;
using System.ComponentModel;
using System.Xml.Serialization;

using System.Reflection;
using System.Windows.Forms;

namespace plexdata.ClipboardViewer
{
    [XmlRoot("Settigns")]
    public class Settigns : ICloneable
    {
        public Settigns()
            : base()
        {
            this.RestoreDefaults();
        }

        [XmlElement("General")]
        public General General { get; set; }

        [XmlElement("HexView")]
        public HexView HexView { get; set; }

        [XmlElement("TextView")]
        public TextView TextView { get; set; }

        [XmlElement("ImageView")]
        public ImageView ImageView { get; set; }

        public static bool Save(string filename, Settigns root)
        {
            bool success = false;
            XmlSerializer serializer = null;
            TextWriter writer = null;

            try
            {
                serializer = new XmlSerializer(typeof(Settigns));
                writer = new StreamWriter(filename);
                serializer.Serialize(writer, root);

                success = true;
            }
            catch (Exception exception)
            {
                Debug.WriteLine(exception);
            }
            finally
            {
                if (writer != null) { writer.Close(); }
                writer = null;
                serializer = null;
            }
            return success;
        }

        public static bool Load(string filename, out Settigns root)
        {
            bool success = false;
            XmlSerializer serializer = null;
            TextReader reader = null;

            root = default(Settigns);

            try
            {
                serializer = new XmlSerializer(typeof(Settigns));
                reader = new StreamReader(filename);
                root = (Settigns)serializer.Deserialize(reader);

                if (root != default(Settigns))
                {
                    root.EnsureScreenLocation();
                    success = true;
                }
            }
            catch (Exception exception)
            {
                Debug.WriteLine(exception);
            }
            finally
            {
                if (reader != null) { reader.Close(); }
                reader = null;
                serializer = null;
            }
            return success;
        }

        public static string Filename
        {
            get
            {
                string path = Path.GetFullPath(Assembly.GetExecutingAssembly().Location);

                return
                    Path.GetDirectoryName(path) +
                    Path.DirectorySeparatorChar +
                    Path.GetFileNameWithoutExtension(path) +
                    ".xml";
            }
        }

        public void RestoreDefaults()
        {
            this.General = new General();
            this.HexView = new HexView();
            this.TextView = new TextView();
            this.ImageView = new ImageView();
        }

        #region Cloneable interface implementation.

        public object Clone()
        {
            Settigns result = new Settigns();
            result.General = (General)this.General.Clone();
            result.HexView = (HexView)this.HexView.Clone();
            result.TextView = (TextView)this.TextView.Clone();
            result.ImageView = (ImageView)this.ImageView.Clone();
            return result;
        }

        #endregion // Cloneable interface implementation.

        private void EnsureScreenLocation()
        {
            Rectangle bounds = Screen.PrimaryScreen.WorkingArea;
            Point location = this.General.Location;
            Size size = this.General.Size;

            foreach (Screen screen in Screen.AllScreens)
            {
                if (screen.Bounds.Contains(new Rectangle(location, size)))
                {
                    return; // Location is still within bounds.
                }
            }

            // Adjust X location to be on the primary screen!

            int x = 0;
            if (location.X < bounds.Left)
            {
                x = bounds.Left;
            }
            else if (location.X + size.Width > bounds.Left + bounds.Right)
            {
                x = bounds.Right - size.Width;
            }
            else
            {
                x = location.X;
            }

            // Adjust Y location to be on the primary screen!

            int y = 0;
            if (location.Y < bounds.Top)
            {
                y = bounds.Top;
            }
            else if (location.Y + size.Height > bounds.Top + bounds.Bottom)
            {
                y = bounds.Bottom - size.Height;
            }
            else
            {
                y = location.Y;
            }

            this.General.Location = new Point(x, y);
        }
    }

    public class General : ICloneable
    {
        public General()
            : base()
        {
            this.RestoreDefaults();
        }

        [XmlElement("Location")]
        public Point Location { get; set; }

        [XmlElement("Size")]
        public Size Size { get; set; }

        [XmlIgnore]
        public Color Foreground { get; set; }

        [XmlElement("Foreground")]
        public int ArgbForeground
        {
            get { return this.Foreground.ToArgb(); }
            set { this.Foreground = Color.FromArgb(value); }
        }

        [XmlIgnore]
        public Color Background { get; set; }

        [XmlElement("Background")]
        public int ArgbBackground
        {
            get { return this.Background.ToArgb(); }
            set { this.Background = Color.FromArgb(value); }
        }

        public void RestoreDefaults()
        {
            this.Location = new Point(100, 100);
            this.Size = new Size(640, 480);
            this.Background = SystemColors.Window;
            this.Foreground = SystemColors.WindowText;
        }

        #region Cloneable interface implementation.

        public object Clone()
        {
            return this.MemberwiseClone();
        }

        #endregion // Cloneable interface implementation.
    }

    public class HexView : ICloneable
    {
        public enum AddressWidth
        {
            [Description("16-bit Address")]
            address16Bit = 4,
            [Description("32-bit Address")]
            address32Bit = 8,
        }

        public enum BlockWidth
        {
            [Description("8-bit Blocks")]
            block8Bit = 2,
            [Description("16-bit Blocks")]
            block16Bit = 4,
            [Description("32-bit Blocks")]
            block32Bit = 8,
        }

        public enum ByteCount
        {
            [Description("8 Bytes")]
            count8Byte = 8,
            [Description("16 Bytes")]
            count16Byte = 16,
            [Description("24 Bytes")]
            count24Byte = 24,
            [Description("32 Bytes")]
            count32Byte = 32,
            [Description("40 Bytes")]
            count40Byte = 40,
            [Description("48 Bytes")]
            count48Byte = 48,
            [Description("56 Bytes")]
            count56Byte = 56,
            [Description("64 Bytes")]
            count64Byte = 64,
        }

        public HexView()
            : base()
        {
            this.RestoreDefaults();
        }

        [XmlElement("HotTrack")]
        public bool HotTrack { get; set; }

        [XmlElement("Address")]
        public AddressWidth Address { get; set; }

        [XmlElement("Block")]
        public BlockWidth Block { get; set; }

        [XmlElement("Bytes")]
        public ByteCount Bytes { get; set; }

        [XmlElement("UpperCase")]
        public bool UpperCase { get; set; }

        public void RestoreDefaults()
        {
            this.HotTrack = true;
            this.UpperCase = true;
            this.Address = AddressWidth.address32Bit;
            this.Block = BlockWidth.block8Bit;
            this.Bytes = ByteCount.count16Byte;
        }

        #region Cloneable interface implementation.

        public object Clone()
        {
            return this.MemberwiseClone();
        }

        #endregion // Cloneable interface implementation.
    }

    public class TextView : ICloneable
    {
        public TextView()
            : base()
        {
            this.RestoreDefaults();
        }

        [XmlElement("Size")]
        public Size Size { get; set; }

        public void RestoreDefaults()
        {
            this.Size = new Size(400, 300);
        }

        #region Cloneable interface implementation.

        public object Clone()
        {
            return this.MemberwiseClone();
        }

        #endregion // Cloneable interface implementation.
    }

    public class ImageView : ICloneable
    {
        public ImageView()
            : base()
        {
            this.RestoreDefaults();
        }

        [XmlElement("Size")]
        public Size Size { get; set; }

        public void RestoreDefaults()
        {
            this.Size = new Size(400, 300);
        }

        #region Cloneable interface implementation.

        public object Clone()
        {
            return this.MemberwiseClone();
        }

        #endregion // Cloneable interface implementation.
    }
}
