﻿/*
 * Copyright (C)  2011  Axel Kesseler
 * 
 * This software is free and you can use it for any purpose. Furthermore, 
 * you are free to copy, to modify and/or to redistribute this software.
 * 
 * In addition, this software is distributed in the hope that it will be 
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * 
 */

using System;
using System.Drawing;
using System.Windows.Forms;
using System.ComponentModel;
using System.Collections.Generic;

namespace plexdata.ClipboardViewer
{
    public partial class SettingsDialog : Form
    {
        private class ComboData<T>
        {
            private T value;
            private string description;

            public ComboData(T value)
            {
                this.value = value;
                this.description = this.GetDescription(value);
            }

            public T Value
            {
                get { return this.value; }
            }

            public string Description
            {
                get { return this.description; }
            }

            public string GetDescription(T value)
            {
                System.Reflection.FieldInfo fi = value.GetType().GetField(value.ToString());
                DescriptionAttribute[] attributes = (DescriptionAttribute[])fi.GetCustomAttributes(typeof(DescriptionAttribute), false);
                return attributes.Length > 0 ? attributes[0].Description : value.ToString();
            }
        }

        private class SelectorData
        {
            private Control value;
            private string description;

            public SelectorData(Control value)
            {
                this.value = value;
                this.description = value.Text;
            }

            public Control Value
            {
                get { return this.value; }
            }

            public string Description
            {
                get { return this.description; }
            }
        }

        private MainForm parent = null;

        public SettingsDialog()
        {
            this.InitializeComponent();
            this.Icon = Icon.FromHandle(plexdata.ClipboardViewer.Properties.Resources.MainIcon.GetHicon());
        }

        public SettingsDialog(MainForm parent, Settigns settigns)
            : this()
        {
            this.parent = parent;
            this.Settigns = settigns;
        }

        private Settigns settigns = new Settigns();
        public Settigns Settigns
        {
            get { return this.settigns; }
            set { this.settigns = (Settigns)value.Clone(); }
        }

        private void SettingsDialog_Load(object sender, EventArgs args)
        {
            this.btnForeground.Color = this.settigns.General.Foreground;
            this.btnForeground.Click += new EventHandler(this.btnForeground_Click);

            this.btnBackground.Color = this.settigns.General.Background;
            this.btnBackground.Click += new EventHandler(this.btnBackground_Click);

            this.chkUpperCase.Checked = this.settigns.HexView.UpperCase;
            this.chkUpperCase.CheckedChanged += new EventHandler(chkUpperCase_CheckedChanged);

            this.chkHotTrack.Checked = this.settigns.HexView.HotTrack;
            this.chkHotTrack.CheckedChanged += new EventHandler(chkHotTrack_CheckedChanged);

            #region Setup address width combo-box.

            this.cmbAddressWidth.DataSource = new List<ComboData<HexView.AddressWidth>>(
                new ComboData<HexView.AddressWidth>[]{
                    new ComboData<HexView.AddressWidth>(HexView.AddressWidth.address16Bit),
                    new ComboData<HexView.AddressWidth>(HexView.AddressWidth.address32Bit)}
                );

            this.cmbAddressWidth.DisplayMember = "Description";
            this.cmbAddressWidth.ValueMember = "Value";
            this.cmbAddressWidth.SelectedValue = this.settigns.HexView.Address;

            // Set handler as last!
            this.cmbAddressWidth.SelectedValueChanged +=
                new EventHandler(this.cmbAddressWidth_SelectedValueChanged);

            #endregion //Setup address width combo-box.

            #region Setup block width combo-box.

            this.cmbBlockWidth.DataSource = new List<ComboData<HexView.BlockWidth>>(
                new ComboData<HexView.BlockWidth>[]{
                    new ComboData<HexView.BlockWidth>(HexView.BlockWidth.block8Bit),
                    new ComboData<HexView.BlockWidth>(HexView.BlockWidth.block16Bit),
                    new ComboData<HexView.BlockWidth>(HexView.BlockWidth.block32Bit)}
                );

            this.cmbBlockWidth.DisplayMember = "Description";
            this.cmbBlockWidth.ValueMember = "Value";
            this.cmbBlockWidth.SelectedValue = this.settigns.HexView.Block;

            // Set handler as last!
            this.cmbBlockWidth.SelectedValueChanged +=
                new EventHandler(this.cmbBlockWidth_SelectedValueChanged);

            #endregion // Setup block width combo-box.

            #region Setup byte count combo-box.

            this.cmbByteCount.DataSource = new List<ComboData<HexView.ByteCount>>(
                new ComboData<HexView.ByteCount>[]{
                    new ComboData<HexView.ByteCount>(HexView.ByteCount.count8Byte),
                    new ComboData<HexView.ByteCount>(HexView.ByteCount.count16Byte),
                    new ComboData<HexView.ByteCount>(HexView.ByteCount.count24Byte),
                    new ComboData<HexView.ByteCount>(HexView.ByteCount.count32Byte),
                    new ComboData<HexView.ByteCount>(HexView.ByteCount.count40Byte),
                    new ComboData<HexView.ByteCount>(HexView.ByteCount.count48Byte),
                    new ComboData<HexView.ByteCount>(HexView.ByteCount.count56Byte),
                    new ComboData<HexView.ByteCount>(HexView.ByteCount.count64Byte)
                });

            this.cmbByteCount.DisplayMember = "Description";
            this.cmbByteCount.ValueMember = "Value";
            this.cmbByteCount.SelectedValue = this.settigns.HexView.Bytes;

            // Set handler as last!
            this.cmbByteCount.SelectedValueChanged +=
                new EventHandler(this.cmbByteCount_SelectedValueChanged);

            #endregion // Setup byte count combo-box.

            #region Setup selector list box.

            this.lstSelector.DataSource = new List<SelectorData>(
                new SelectorData[]{
                    new SelectorData(this.grpGeneral),
                    new SelectorData(this.grpDisplay),
                });

            this.lstSelector.SelectedValueChanged +=
                new EventHandler(this.lstSelector_SelectedValueChanged);

            this.lstSelector.DisplayMember = "Description";
            this.lstSelector.ValueMember = "Value";

            this.lstSelector.SelectedValue = this.grpDisplay;

            #endregion // Setup selector list box.
        }

        private void btnApply_Click(object sender, EventArgs args)
        {
            if (this.parent != null)
            {
                this.parent.ApplySettings(this.Settigns);
            }
        }

        private void btnForeground_Click(object sender, EventArgs args)
        {
            ColorDialog dialog = new ColorDialog();
            dialog.Color = this.btnForeground.Color;
            if (DialogResult.OK == dialog.ShowDialog(this))
            {
                this.btnForeground.Color = dialog.Color;
                this.settigns.General.Foreground = dialog.Color;
            }
        }

        private void btnBackground_Click(object sender, EventArgs args)
        {
            ColorDialog dialog = new ColorDialog();
            dialog.Color = this.btnBackground.Color;
            if (DialogResult.OK == dialog.ShowDialog(this))
            {
                this.btnBackground.Color = dialog.Color;
                this.settigns.General.Background = dialog.Color;
            }
        }

        private void lstSelector_SelectedValueChanged(object sender, EventArgs args)
        {
            Control selected = (this.lstSelector.SelectedValue as Control);

            foreach (SelectorData current in this.lstSelector.Items)
            {
                if (current.Value == selected)
                {
                    current.Value.Show();
                }
                else
                {
                    current.Value.Hide();
                }
            }
        }

        private void cmbAddressWidth_SelectedValueChanged(object sender, EventArgs args)
        {
            this.settigns.HexView.Address =
                ((ComboData<HexView.AddressWidth>)this.cmbAddressWidth.SelectedItem).Value;
        }

        private void cmbBlockWidth_SelectedValueChanged(object sender, EventArgs args)
        {
            this.settigns.HexView.Block =
                ((ComboData<HexView.BlockWidth>)this.cmbBlockWidth.SelectedItem).Value;
        }

        private void cmbByteCount_SelectedValueChanged(object sender, EventArgs args)
        {
            this.settigns.HexView.Bytes =
                ((ComboData<HexView.ByteCount>)this.cmbByteCount.SelectedItem).Value;
        }

        private void chkUpperCase_CheckedChanged(object sender, EventArgs args)
        {
            this.settigns.HexView.UpperCase = this.chkUpperCase.Checked;
        }

        private void chkHotTrack_CheckedChanged(object sender, EventArgs args)
        {
            this.settigns.HexView.HotTrack = this.chkHotTrack.Checked;
        }
    }
}

// EOF