﻿/*
 * Copyright (C)  2011  Axel Kesseler
 * 
 * This software is free and you can use it for any purpose. Furthermore, 
 * you are free to copy, to modify and/or to redistribute this software.
 * 
 * In addition, this software is distributed in the hope that it will be 
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * 
 */

using System;
using System.IO;
using System.Drawing;
using System.Reflection;
using System.Diagnostics;
using System.Windows.Forms;
using System.Collections.Generic;

namespace plexdata.ClipboardViewer
{
    public partial class TextViewDialog : Form
    {
        private List<ExportFilter> filters = new List<ExportFilter>();

        public TextViewDialog()
        {
            this.InitializeComponent();
            this.Icon = Icon.FromHandle(plexdata.ClipboardViewer.Properties.Resources.MainIcon.GetHicon());
        }

        public TextViewDialog(MainForm parent)
            : this()
        {
            if (parent != null)
            {
                parent.ClipboardChanged += new EventHandler<EventArgs>(this.OnClipboardChanged);
            }
        }

        private void TextViewDialog_Load(object sender, EventArgs args)
        {
            this.LoadData();
        }

        private void OnClipboardChanged(object sender, EventArgs args)
        {
            this.LoadData();
        }

        private void btnClose_Click(object sender, EventArgs args)
        {
            this.Close();
        }

        private void btnExport_Click(object sender, EventArgs args)
        {
            SaveFileDialog dialog = new SaveFileDialog();
            dialog.Filter = ExportFilter.BuildFilters(this.filters);
            dialog.FilterIndex = this.GetFileIndex();
            dialog.InitialDirectory = Path.GetDirectoryName(Assembly.GetExecutingAssembly().Location);
            dialog.RestoreDirectory = true;

            if (DialogResult.OK == dialog.ShowDialog(this))
            {
                try
                {
                    using (StreamWriter writer = File.CreateText(dialog.FileName))
                    {
                        writer.Write(
                            Clipboard.GetText(
                                this.filters[dialog.FilterIndex - 1].TextFormat));
                        writer.Close();
                    }
                }
                catch (Exception exception)
                {
                    MessageBox.Show(
                        this, "File couldn't be saved!\n\n" + exception.ToString(),
                        "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
        }

        private void btnText_CheckedChanged(object sender, EventArgs args)
        {
            if (this.btnText.Checked)
            {
                this.textViewer.SetText((this.btnText.Tag as string));
            }
        }

        private void btnRtf_CheckedChanged(object sender, EventArgs args)
        {
            if (this.btnRtf.Checked)
            {
                this.textViewer.SetText((this.btnRtf.Tag as string), true);
            }
        }

        private void btnHtml_CheckedChanged(object sender, EventArgs args)
        {
            if (this.btnHtml.Checked)
            {
                this.textViewer.SetText((this.btnHtml.Tag as string));
            }
        }

        private void LoadData()
        {
            this.btnText.Tag = null;
            this.btnRtf.Tag = null;
            this.btnHtml.Tag = null;

            this.filters.Clear();

            if (Clipboard.ContainsText(TextDataFormat.Text))
            {
                this.btnText.Tag = Clipboard.GetText(TextDataFormat.Text);
                this.filters.Add(new ExportFilter("Plain Text Files (*.txt)", "*.txt", TextDataFormat.Text));
            }

            if (Clipboard.ContainsText(TextDataFormat.Rtf))
            {
                this.btnRtf.Tag = Clipboard.GetText(TextDataFormat.Rtf);
                this.filters.Add(new ExportFilter("Rich Text Files (*.rtf)", "*.rtf", TextDataFormat.Rtf));
            }

            if (Clipboard.ContainsText(TextDataFormat.Html))
            {
                this.btnHtml.Tag = Clipboard.GetText(TextDataFormat.Html);
                this.filters.Add(new ExportFilter("HTML Files (*.html)", "*.html", TextDataFormat.Html));
            }

            this.filters.Add(ExportFilter.AllFiles());

            this.btnText.Enabled = (this.btnText.Tag != null);
            this.btnRtf.Enabled = (this.btnRtf.Tag != null);
            this.btnHtml.Enabled = (this.btnHtml.Tag != null);

            if (this.btnHtml.Checked)
            {
                this.btnHtml.Checked = (this.btnHtml.Tag != null);
                this.textViewer.SetText((this.btnHtml.Tag as string));
            }
            else if (this.btnRtf.Checked)
            {
                this.btnRtf.Checked = (this.btnRtf.Tag != null);
                this.textViewer.SetText((this.btnRtf.Tag as string), true);
            }
            else
            {
                this.btnText.Checked = (this.btnText.Tag != null);
                this.textViewer.SetText((this.btnText.Tag as string));
            }

            this.btnExport.Enabled =
                this.btnText.Enabled || this.btnRtf.Enabled || this.btnHtml.Enabled;
        }

        private int GetFileIndex()
        {
            for (int index = 0; index < this.filters.Count; index++)
            {
                TextDataFormat format = this.filters[index].TextFormat;
                if (format == TextDataFormat.Html && this.btnHtml.Checked)
                {
                    return index + 1;
                }
                else if (format == TextDataFormat.Rtf && this.btnRtf.Checked)
                {
                    return index + 1;
                }
                else if (format == TextDataFormat.Text && this.btnText.Checked)
                {
                    return index + 1;
                }
            }
            return this.filters.Count;
        }
    }
}

// EOF